% Tests the largest eigenvalue of a beta-Laguerre matrix vs
% hypergeometric formula (cdfMaxEigBetaLaguerre)
%
% Copyright (c) Plamen Koev, Massachusetts Institute of Technology
% May 17, 2005. See COPYRIGHT.TXT for details.


%set(0,'defaultaxesfontsize',12,'defaultaxeslinewidth',0.7, ...
%      'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
n=3; beta=2; m=3; % we must have n>m-1
 
nn=10000;               % # of trials
r=zeros(1,nn);

bins=120;

   clf
   hold off

   a=beta*n/2;

   d=zeros(m,nn);
   l=zeros(m-1,nn);
   for i=1:m-1
      d(i+1,:)=sqrt(chi2rnd(2*a-beta*i,1,nn));
      l(i,:)=sqrt(chi2rnd(beta*(m-i),1,nn));
   end
   d(1,:)=sqrt(chi2rnd(2*a,1,nn));
   
   for j=1:nn
      z=svd(diag(d(:,j))+diag(l(:,j),1));
      r(j)=z(1)^2;        % largest e.value
   end
   
   x=[0:2:bins/4];

   z=cdfMaxEigBetaLaguerre(95,beta,m,n,x);
   
plot(x,z,'x'); hold on
   
plot(sort(r),(1:nn)/nn)

axis([0 bins/4 0 1.1]);

xlabel('x');
s=strcat('cdf of \lambda_m_a_x of (',...
num2str(n),'\times',num2str(n),'), ',num2str(beta),'-Laguerre, a=',...
num2str(a));

title(s);
legend('predicted','experimental',4);
