% function f=pdfMinEigBetaLaguerre(M,beta,n,k,x)
% 
% Computes the pdf of the smallest eigenvalue of a beta-Laguerre matrix vs 
% hypergeometric formula.
%
% See Ph.D thesis of Ioana Dumitriu, p. 146 for theoretical background.
%
% Copyright (c) Plamen Koev, Massachusetts Institute of Technology.
% May 17, 2005

function f=pdfMinEigBetaLaguerre(M,beta,n,a,x)

k=a-beta*(n-1)/2-1;

f=x.^(k*n).*exp(-n*x/2).*mhgi(M,2/beta,[-k,beta*n/2+1],[],n-1,-2./x);
  
% need to scale, so that the area is 1:

f=f/sum(f)/(x(2)-x(1));
