% function ss=pdfTraceWishart(MAX1,u,Sigma,M)
% computes the pdf of tr A, where A is Wishart_n(M,\Sigma)
%
% Although Sigma is in practice a matrix, this function only 
% takes its eigenvalues as a parameter, i.e., Sigma must be a vector
% and then we have n=length(Sigma)
%
% by Plamen Koev, May 2004

function s=pdfTraceWishart(MAX1,uu,Sigma,MM)
  global MAX lambda xn XY Lp n alpha ss sh heap block M u LL

  alpha=2; lma=[]; MAX=MAX1; M=MM; u=uu;
  n=length(Sigma);
  
  SigmaS=sort(Sigma);
  LL=2*SigmaS(1)*SigmaS(n)/(SigmaS(1)+SigmaS(n));
  clear SigmaS;
  x=1-LL./Sigma;
  
  n=length(x);
  lambda=floor(MAX./[1:n]);
  if MAX==0, MAX=sum(lambda); end
  
  Lp=0;
  while (length(lambda)>Lp)&(lambda(Lp+1)>0)
     Lp=Lp+1;
  end % number of parts in l
  lambda=lambda(1:Lp);
  
  % figure out number of partitions lambda with |lambda|<=MAX
  f=zeros(MAX+1);
  f(1,1)=1;
  for i=1:MAX
     for k=1:i
       f(i+1,k+1)=f(i+1,k)+f(i-k+1,min(i-k,k)+1);
     end
  end
  i=sum(diag(f));
  clear f;
  
  block = n+1+Lp;
  heap=block*(lambda(1)+1)+1;
  sh=zeros(1,i*block);
  sh(1+Lp+[1:n])=1;
  
  xn=ones(n,MAX+1);
 
  if size(x,2)>1, x=x'; end
  for i=2:MAX+1
     xn(:,i)=xn(:,i-1).*x;
  end
  
%  ss=1;
  r=M*n/2;
  ss=u.^(r-1)/(2*LL)^r/gamma(r);

  summation(1,zeros(1,Lp),zeros(1,lambda(1)),1,1);
  s=ss.*exp(-u/2/LL).*prod(Sigma/LL)^(-M/2);
   
  clear MAX xn Lp n alpha lambda sh heap block
  clear functions
  clear global
  
function summation(i,l,lc,z,w);
  global MAX Lp xn n lambda alpha ss sh heap block M u LL

  suml=sum(l(1:i-1));

  m=min(lambda(i),MAX-suml);
  if i>1
     m=min(m,l(i-1));
  end
     
  for ii=1:m
     if (ii==1)&(i>1)
       sh(w)=heap;
       w=heap;
       heap=heap+m*block;
     else
       w=w+block;
     end
     
     l(i)=l(i)+1;
     lc(l(i))=lc(l(i))+1;        % update conjugate partition
        
     for j=1:lc(1)
       if j<lc(1), lj1=l(j+1);
       else lj1=0;
       end
       if l(j)>lj1 
         mm=(l(1)-(j==1))*block+1;
         
         for jj=2:lc(1)
            if l(jj)>(jj==j) 
               mm=sh(mm)+(l(jj)-1-(jj==j))*block;
            end
         end
         sh(w+1+j-1)=mm;
       end
     end
        
     dn=1;
     c=-(i-1)/alpha+l(i)-1;
     z=z*(M/2+c);
        
     d=l(i)*alpha-i;             % update j_lambda
     for j=1:l(i)-1
        e=d-j*alpha+lc(j);
        g=e+1;
        z=z*(g-alpha)*e;
        dn=dn*g*(e+alpha);
     end
     for j=1:i-1
        f=l(j)*alpha-j-d;
        g=f+alpha;
        e=f*g;
        z=z*(e-f);
        dn=dn*(g+e);
     end
     z=z/dn;
        
     e=w+1+Lp;
     if lc(1)==1
        sh(e)=xn(1,2)'*sh(e-block)*(1+alpha*(l(1)-1));
     end
        
     jack1(0,l,lc,l,lc,1,0,w,w);
     k=sum(l);
     r=M*n/2+k;
     g=u.^(r-1)/(2*LL)^r/gamma(r);
     ss=ss+g*z*sh(e+n-1);
     if i<Lp, summation(i+1,l,lc,z,w); end;
  end
  
function jack1(k,l,lc,m,mc,aux,sumlm,wl,wm);
   global xn yn Lp n alpha sh block MAX
   
   start=lc(1)+(lc(1)==1);
    
   i=k+(k==0);
   wlms=wl+Lp+start;
   wlmn=wl+Lp+n;
   
   cpc=mc; cp=m;
   while i<=mc(1)
      mi1=0; if i<Lp mi1=m(i+1); end;
      if m(i)>mi1
        cpc(m(i))=cpc(m(i))-1;
        cp(i)=cp(i)-1;
        
        t=l(i)-m(i);
        aux1=aux*(1+alpha*t);
        dn=t+1;
        t=i-alpha*m(i);
        for r=1:i-1
           q0=t-r;
           q1=q0+alpha*(l(r)+1);
           q2=q0+alpha*m(r);
           aux1=aux1*(q1+1-alpha)*(q2+alpha);
           dn=dn*q1*q2;
        end
        
        for r=1:m(i)-1
           q1=mc(r)-t-alpha*r;
           aux1=aux1*(q1+alpha);
           dn=dn*q1;
        end
        aux1=aux1/dn;
        
        wc=sh(wm+i);
        if cp(i)>0
          jack1(i,l,lc,cp,cpc,aux1,sumlm+1,wl,wc);
        else
          sh([wlms:wlmn])=sh([wlms:wlmn])+...
                       sh(wc+Lp+1-1+[start-1:n-1])*aux1.*xn(start:n,sumlm+2)';
        end
        
        cpc(m(i))=cpc(m(i))+1;
        cp(i)=cp(i)+1;
      end
      i=i+1;
   end
     if k==0
       for j=wlms:wlmn, sh(j)=sh(j)+sh(j-1); end;
     else 
      sh([wlms:wlmn])=sh([wlms:wlmn])+...
         sh(wm+Lp+[start-1:n-1])*aux.*xn(start:n,sumlm+1)'; 
     end
                    
